///////////////////////////////////////////////////////////////////////////////
//                                                         
// CegoGroupCursor.cc
// -------------------
// Cego order cursor class implementation
//      
// Design and Implementation by Bjoern Lemke
//     
// (C)opyright 2000-20010 Bjoern Lemke
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; see the file COPYING.  If not, write to
// the Free Software Foundation, 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.
//
// IMPLEMENTATION MODULE
//
// Class: CegoGroupCursor
// 
// Description: A cursor, to trace group space tables
//
// Status: QG-2.6
//
///////////////////////////////////////////////////////////////////////////////

#include "CegoGroupCursor.h"
#include "CegoGroupNode.h"
#include "CegoDefs.h"

CegoGroupCursor::CegoGroupCursor(ListT<CegoField>& schema, CegoGroupNode *pRoot)
{
    _pRoot = pRoot;
    _pAct = pRoot;
    
    int idx=0;
    CegoField *pF = schema.First();
    while ( pF )
    {
	if ( pF->getTableAlias() == Chain(AVG_GROUP) )
	{
	    _avgIdxList.Insert(idx);
	}
	pF = schema.Next();
	idx++;
    }
}

CegoGroupCursor::~CegoGroupCursor()
{
}

bool CegoGroupCursor::getFirst(ListT<CegoField>& fl)
{
    return getMostLeft(fl);
}

bool CegoGroupCursor::getMostLeft(ListT<CegoField>& fl)
{
    if ( _pAct == 0 )
	return false;
    
    while ( _pAct->Left() != 0 )
    {
	_pAct = _pAct->Left();
    }

    fl = _pAct->getKey() + _pAct->getGrouping(); 

    int *pI = _avgIdxList.First();
    while ( pI )
    {
	CegoFieldValue fv = fl[*pI].getValue();
	CegoFieldValue countValue = fl[fl.Size() - 1 ].getValue();
	fv = fv / countValue;
	fl[*pI].setValue(fv);
	pI = _avgIdxList.Next();
    }

    return true;
    
}

bool CegoGroupCursor::getNext(ListT<CegoField>& fl)
{

    if ( _pAct == 0 )
	return false;
    
    if ( _pAct->Right() == 0 )
    {

	CegoGroupNode* pPrev = _pAct;

	_pAct = _pAct->Parent();	
	if ( _pAct == 0 )
	    return false;

	while ( _pAct->Right() == pPrev )
	{
	    pPrev = _pAct;
	    _pAct = _pAct->Parent();
	    if ( _pAct == 0 )
		return false;
	}
	
	fl = _pAct->getKey() + _pAct->getGrouping(); 

	int *pI = _avgIdxList.First();
	while ( pI )
	{
	    CegoFieldValue fv = fl[*pI].getValue();
	    CegoFieldValue countValue = fl[fl.Size() - 1 ].getValue();
	    fv = fv / countValue;
	    fl[*pI].setValue(fv);
	    pI = _avgIdxList.Next();
	}
		
	return true;
    }
    else
    {	
	_pAct = _pAct->Right();
	return getMostLeft(fl);	
    }
}

void CegoGroupCursor::abort()
{
    _pAct = _pRoot;
}
