///////////////////////////////////////////////////////////////////////////////
//                                                         
// CegoOrderCursor.cc
// -------------------
// Cego order cursor class implementation
//      
// Design and Implementation by Bjoern Lemke
//     
// (C)opyright 2000-2010 Bjoern Lemke
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; see the file COPYING.  If not, write to
// the Free Software Foundation, 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.
//
// IMPLEMENTATION MODULE
//
// Class: CegoOrderCursor
// 
// Description: A cursor, to trace order space tables
//
// Status: QG-2.6
//
///////////////////////////////////////////////////////////////////////////////


// cego includes
#include "CegoOrderCursor.h"
#include "CegoOrderNode.h"

CegoOrderCursor::CegoOrderCursor(CegoOrderNode *pRoot)
{
    _pRoot = pRoot;
    _pAct = pRoot;
}

CegoOrderCursor::~CegoOrderCursor()
{
}

bool CegoOrderCursor::getFirst(ListT<CegoField>& fl)
{
    return getMostLeft(fl);
}

bool CegoOrderCursor::getMostLeft(ListT<CegoField>& fl)
{
    if ( _pAct == 0 )
	return false;
    
    while ( _pAct->Left() != 0 )
    {
	_pAct = _pAct->Left();
    }

    fl = _pAct->getTuple();

    return true;
    
}

bool CegoOrderCursor::getNext(ListT<CegoField>& fl)
{

    if ( _pAct == 0 )
	return false;
    
    if ( _pAct->Right() == 0 )
    {

	CegoOrderNode* pPrev = _pAct;

	_pAct = _pAct->Parent();	
	if ( _pAct == 0 )
	    return false;

	while ( _pAct->Right() == pPrev )
	{
	    pPrev = _pAct;
	    _pAct = _pAct->Parent();
	    if ( _pAct == 0 )
		return false;
	}
	
	fl = _pAct->getTuple();
		
	return true;
    }
    else
    {	
	_pAct = _pAct->Right();
	return getMostLeft(fl);	
    }
}

void CegoOrderCursor::abort()
{
    _pAct = _pRoot;
}
